#!/bin/sh

if [ $SHLVL -gt 2 ] ; then
	# The above test might fail if the Tracker has been started from a shell.
	if [ $PWD != /boot/home ] ; then
		alert --stop "Please run the Installer through the Tracker." &
		exit
	fi
fi

INSTALLDIR="/boot/home/config/bin"
APPNAME="BackOnTrack"
BOOTSCRIPTDIR="/boot/home/config/boot"
STARTKEYWORD="# -- START BackOnTrack -- DO NOT ALTER THIS LINE --"
ENDKEYWORD="# --  END  BackOnTrack -- DO NOT ALTER THIS LINE --"

if [ $(uname -m) = "BePC" ] ; then
	PLATFORM="x86"
	MACHINETYPE="BeOS Intel x86"
else
	PLATFORM="ppc"
	MACHINETYPE="BeOS PowerPC"
fi

RESULT=$(alert --info "$APPNAME Installer

Thank you for downloading $APPNAME. This installer will automatically install and \
configure $APPNAME for $MACHINETYPE. Do you wish to proceed with this operation?" \
"No" "Yes")

if [ "$RESULT" = "No" ] ; then
	exit
fi

# Find our parent directory
TOMEDIR=$(dirname "$0")

cd "$TOMEDIR/obj.$PLATFORM"

if [ ! -e "$APPNAME" ] ; then
	alert --stop "There was an error in the installation process.

The application binary for $APPNAME cannot be found. Try downloading a new archive \
and install from that. If this error occurs again, please contact the author.

The installer will now quit. No changes have been made to your system." &
	exit
fi

# Look for a previous version
DESTINATION=$(find "$INSTALLDIR" -name "$APPNAME")

if [ "$DESTINATION" ] ; then
	RESULT=$(alert --warning "A version of $APPNAME is already installed.
Do you wish to replace it?" "No" "Yes")
	if [ $RESULT = "No" ] ; then
		alert --info "Installation halted. No changes have been made to your system." &
		exit
	fi
fi

cp -f "$APPNAME" "$INSTALLDIR"
mimeset -f "$INSTALLDIR/$APPNAME"

RESULT=$(alert --info "Would you like to have $APPNAME configured to automatically start \
up and shut down?

(Requires modification to your UserBootscript and UserShutdownScript. Your current \
scripts will be backed up.)" "No" "Yes")

if [ $RESULT = "Yes" ] ; then
	cd "$BOOTSCRIPTDIR"
	
	if [ ! -e UserBootscript ] ; then
		echo '#!/bin/sh' >UserBootscript
	else
		cp -f UserBootscript UserBootscript~
	fi
	RESULT=$(grep -x "$STARTKEYWORD" UserBootscript)
	if [ ! $RESULT ] ; then
		cat >>UserBootscript <<EOF


$STARTKEYWORD
if [ -e $INSTALLDIR/$APPNAME ] ; then
	$INSTALLDIR/$APPNAME &
fi
$ENDKEYWORD
EOF
	fi
	
	if [ ! -e UserShutdownScript ] ; then
		echo '#!/bin/sh' >UserShutdownScript
	else
		cp -f UserShutdownScript UserShutdownScript~
	fi
	RESULT=$(grep -x "$STARTKEYWORD" UserShutdownScript)
	if [ ! $RESULT ] ; then
		cat >>UserShutdownScript <<EOF


$STARTKEYWORD
if [ -e $INSTALLDIR/$APPNAME ] ; then
	$INSTALLDIR/$APPNAME --quit
fi
$ENDKEYWORD
EOF
	fi
fi

RESULT=$(alert --info "Would you like $APPNAME to be run right now?" "No" "Yes")
if [ $RESULT = "Yes" ] ; then
	$INSTALLDIR/$APPNAME --quit
	$INSTALLDIR/$APPNAME &
fi

cd "$TOMEDIR"
cd ../

RESULT=$(alert --info "Installation complete. Would you like to view the documentation \
for $APPNAME?" "No" "Yes")

if [ $RESULT = "Yes" ] ; then
	/system/Tracker "$PWD/README.html" &
fi
